@@ -2,6 +2,7 @@
 use strict;
 use AnyEvent;
 use Search::Elasticsearch::Async;
+use Dancer::SearchApp::Defaults 'default_index';
 use Promises qw[collect deferred];
 
 use Getopt::Long;
@@ -17,7 +18,7 @@ use URI::file;
 use POSIX 'strftime';
 
 use Dancer::SearchApp::IndexSchema qw(create_mapping find_or_create_index %indices %analyzers );
-use Dancer::SearchApp::Utils qw(synchronous);
+use Dancer::SearchApp::Utils qw(await);
 
 #use lib 'C:/Users/Corion/Projekte/Apache-Tika/lib';
 use CORION::Apache::Tika::Server;
@@ -45,9 +46,10 @@ GetOptions(
 );
 $config_file ||= 'fs-import.yml';
 
-my $config = LoadFile($config_file)->{fs};
+my $config = LoadFile($config_file);
 
-my $index_name = 'dancer-searchapp';
+my $index_name = $config->{index} || default_index;
+$config = $config->{fs};
 
 my $e = Search::Elasticsearch::Async->new(
     nodes => [
@@ -71,7 +73,7 @@ my $tika= CORION::Apache::Tika::Server->new(
 $tika->launch;
 
 my $ok = AnyEvent->condvar;
-my $info = synchronous $e->cat->plugins;
+my $info = await $e->cat->plugins;
 
 # Koennen wir ElasticSearch langdetect als Fallback nehmen?
 my $have_langdetect = $info =~ /langdetect/i;
@@ -96,18 +98,18 @@ use vars qw(%analyzers);
 if( $force_rebuild ) {
     print "Dropping indices\n";
     my @list;
-    synchronous $e->indices->get({index => ['*']})->then(sub{
+    await $e->indices->get({index => ['*']})->then(sub{
         @list = grep { /^\Q$index_name/ } sort keys %{ $_[0]};
     });
 
-    synchronous collect( map { my $n=$_; $e->indices->delete( index => $n )->then(sub{warn "$n dropped" }) } @list )->then(sub{
+    await collect( map { my $n=$_; $e->indices->delete( index => $n )->then(sub{warn "$n dropped" }) } @list )->then(sub{
         warn "Index cleanup complete";
         %indices = ();
     });
 };
 
 print "Reading ES indices\n";
-synchronous $e->indices->get({index => ['*']})->then(sub{
+await $e->indices->get({index => ['*']})->then(sub{
     %indices = %{ $_[0]};
 });
 
@@ -198,7 +200,8 @@ sub get_file_info {
     my( $file ) = @_;
     my %res;
     $res{ url } = URI::file->new( $file )->as_string;
-    # Involve Apache::Tika here
+    $res{ folder } = "" . $file->dir;
+    $res{ folder } =~ s![\\/ ]! !g;
     
     eval {
         $info = $tika->get_all( $file );
@@ -215,6 +218,8 @@ sub get_file_info {
         
         $res{ mime_type } = $meta->{"Content-Type"};
         
+        # This should be general dispatching
+        # so the IMAP import can benefit from that
         if( $res{ mime_type } =~ m!^audio/mpeg$! ) {
             require MP3::Tag;
             my $mp3 = MP3::Tag->new($file);
@@ -222,10 +227,23 @@ sub get_file_info {
             $res{ title } = $title || $file->basename;
             $res{ author } = $artist;
             $res{ language } = 'en'; # ...
-            $res{ content } = join "-", $artist, $album, $track, $comment, $genre;
+            $res{ content } = join "-", $artist, $album, $track, $comment, $genre, $file->basename, 'mp3';
             # We should also calculate the duration here, and some more information
             # to generate an "HTML" page for the file
+            # These special pages should be named "cards"
             
+        } elsif( $res{ mime_type } =~ m!^image/.*$! ) {
+            require Image::ExifTool;
+            my $info = Image::ExifTool->new;
+            $info->ExtractInfo("$file");
+            
+            $res{ title } = $info->GetValue( 'Title' ) || $file->basename;
+            $res{ author } = $info->GetValue( 'Author' );
+            $res{ language } = 'en'; # ...
+            $res{ content } = join "-", map { $_ => $info->GetValue($_) } $info->GetFoundTags('File'), $file->basename;
+            # We should also generate/store a (tiny) thumbnail here
+            # to generate an "HTML" page for the file
+
         } else {
             
             # Just use what Tika found
@@ -270,6 +288,9 @@ sub detect_language {
     $res
 }
 
+sub url_stored {
+}
+
 if( @ARGV) {
     $config->{directories} = [@ARGV];
 };
