@@ -7,10 +7,12 @@ use URI::Escape 'uri_unescape';
 use URI::file;
 #use Search::Elasticsearch::TestServer;
 
+use Dancer::SearchApp::Defaults 'default_index';
+
 use Dancer::SearchApp::Entry;
 
 use vars qw($VERSION $es %indices);
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 =head1 NAME
 
@@ -20,6 +22,8 @@ Dancer::SearchApp - A simple local search engine
 
 =head1 QUICKSTART
 
+Also see L<Dancer::SearchApp::Installation>.
+
   cpanm --look Dancer::SearchApp
   
   # Install prerequisites
@@ -56,6 +60,9 @@ Configuration happens through config.yml
     home: "./elasticsearch-2.1.1/"
     index: "dancer-searchapp"
 
+The Elasticsearch instance to used can also be passed in C<%ENV>
+as C<SEARCHAPP_ES_NODES>.
+
 =cut
 
 use Data::Dumper;
@@ -63,18 +70,14 @@ $Data::Dumper::Sortkeys = 1;
 
 sub search {
     if( ! $es ) {
-        #my $nodes;
-        #if( config->{elastic_search}) {
-        #    my $es_home = config->{elastic_search}->{home};
-        #    warning "Starting local ElasticSearch instance in $es_home";
-        #    $server ||= Search::Elasticsearch::TestServer->new(
-        #        es_home   => $es_home,
-        #    );
-
-            #$nodes = $server->start;
-        #};
+        my $nodes;
+        if( $nodes = $ENV{SEARCHAPP_ES_NODES} ) {
+            $nodes = [ split /;/, $nodes ];
+        } else {
+            $nodes = config->{nodes} || [];
+        };
         $es = Search::Elasticsearch->new(
-            nodes => config->{nodes} || [],
+            nodes => $nodes,
         );
     };
     
@@ -91,22 +94,27 @@ get '/' => sub {
     my $statistics;
     my $results;
     
-    my $from = params->{'from'} || '';
+    my $from = params->{'from'} || 0;
     $from =~ s!\D!!g;
-    $from ||= 0;
-    my $size = params->{'size'} || '';
+    my $size = params->{'size'} || 25;
     $size =~ s!\D!!g;
-    $size ||= 10;
+    my $search_term = params->{'q'};
     
-    if( defined params->{'q'}) {
+    if( defined $search_term) {
         
-        warning "Reading ES indices\n";
+        #warning "Reading ES indices\n";
         %indices = %{ search->indices->get({index => ['*']}) };
-        warning $_ for sort keys %indices;
+        #warning $_ for sort keys %indices;
 
+        my @restrict_type;
+        my $type;
+        if( $type = params->{'type'} and $type =~ m!([a-z0-9+-]+)/[a-z0-9+-]+!i) {
+            #warn "Filtering for '$type'";
+            @restrict_type = (filter => { term => { mime_type => $type }});
+        };
+        
         # Move this to an async query, later
-        my $search_term = params->{'q'};
-        my $index = config->{elastic_search}->{index};
+        my $index = config->{elastic_search}->{index} || default_index;
         $results = search->search(
             # Wir suchen in allen Sprachindices
             index => [ grep { /^\Q$index\E/ } sort keys %indices ],
@@ -114,9 +122,14 @@ get '/' => sub {
                 from => $from,
                 size => $size,
                 query => {
-                    query_string => {
-                        query => $search_term,
-                        fields => ['title','content', 'author'] #'creation_date'] 
+                    filtered => {
+                        query => {
+                            query_string => {
+                                query => $search_term,
+                                fields => ['title','folder','content', 'author'] #'creation_date'] 
+                            },
+                        },
+                        @restrict_type,
                     },
                 },
                 sort => {
@@ -161,36 +174,20 @@ get '/' => sub {
     template 'index', {
             results => ($results ? $results->{hits} : undef ),
             params => {
-                q=> params()->{q},
+                q    => $search_term,
                 from => $from,
                 size => $size,
             },
     };
 };
 
-sub retrieve {
-    my( $index, $type, $id ) = @_;
-    my $document;
-    if( eval {
-        $document = search->get(index => $index, type => $type, id => $id);
-        1
-    }) {
-        my $res = Dancer::SearchApp::Entry->from_es($document);
-        return $res
-    } else {
-        warn "$@";
-    };
-    # Not found in the Elasticsearch index
-    return undef
-}
-
 # Show (cached) elements
 get '/cache/:index/:type/:id' => sub {
     my $index = params->{index};
     my $type = params->{type};
     my $id = uri_unescape( params->{id} );
     my $document = retrieve($index,$type,$id);
-    warn $document->basic_mime_type;
+    #warn $document->basic_mime_type;
     
     $document->{type} = $type;
     $document->{index} = $index;
@@ -238,18 +235,38 @@ SORRY
     }
 };
 
+sub retrieve {
+    my( $index, $type, $id ) = @_;
+    my $document;
+    if( eval {
+        $document = search->get(index => $index, type => $type, id => $id);
+        1
+    }) {
+        my $res = Dancer::SearchApp::Entry->from_es($document);
+        return $res
+    } else {
+        warn "$@";
+    };
+    # Not found in the Elasticsearch index
+    return undef
+}
+
 get '/open/:index/:type/:id' => sub {
     my $index = params->{index};
     my $type = params->{type};
     my $id = uri_unescape params->{id};
     my $document = retrieve($index,$type,$id);
-    my $local = URI::file->new( $id )->file;
-    
-    reproxy( $document, $local, 'Attachment',
-        index => $index,
-        type => $type,
-    );
-    
+    if( $type eq 'http' ) {
+        return
+            redirect $id
+    } else {
+        my $local = URI::file->new( $id )->file;
+        return
+        reproxy( $document, $local, 'Attachment',
+            index => $index,
+            type => $type,
+        );
+    }
 };
 
 get '/inline/:index/:type/:id' => sub {
@@ -257,7 +274,13 @@ get '/inline/:index/:type/:id' => sub {
     my $type = params->{type};
     my $id = uri_unescape params->{id};
     my $document = retrieve($index,$type,$id);
-    my $local = URI::file->new( $id )->file;
+    
+    my $local;
+    if( 'http' eq $type ) {
+        $document->content
+    } else {
+        $local = URI::file->new( $id )->file;
+    };
     
     reproxy( $document, $local, 'Inline',
         index => $index,
@@ -311,7 +334,9 @@ L<https://perlmonks.org/>.
 
 I've given a talk about this module at Perl conferences:
 
-L<German Perl Workshop 2016, German|http://corion.net/talks/dancer-searchapp/dancer-searchapp.html>
+L<German Perl Workshop 2016, German|http://corion.net/talks/dancer-searchapp/dancer-searchapp.de.html>
+
+L<German Perl Workshop 2016, English|http://corion.net/talks/dancer-searchapp/dancer-searchapp.en.html>
 
 =head1 BUG TRACKER
 
