@@ -14,10 +14,12 @@ Dancer::SearchApp::IndexSchema - schema definition for the Elasticsearch index
 
 =cut
 
-use vars qw(@EXPORT_OK $VERSION);
-$VERSION = '0.03';
+use vars qw(@EXPORT_OK $VERSION @types);
+$VERSION = '0.04';
 @EXPORT_OK = qw(create_mapping multilang_text find_or_create_index %indices %analyzers );
 
+@types = (qw(file mail http));
+
 # Datenstruktur für ES Felder, deren Sprache wir nicht kennen
 sub multilang_text($$) {
     my($name, $analyzer)= @_;
@@ -67,7 +69,14 @@ sub create_mapping {
             "title"      => multilang_text('title',$analyzer),
             "author"     => multilang_text('author', $analyzer),
             "content"    => multilang_text('content',$analyzer),
-            'mime_type'  => { type => "string" }, # text/html etc.
+            "folder"     => {
+                  "type" => "string",
+                  "analyzer" => $analyzer,
+                  # Some day I'll know how to have a separate tokenizer per-field
+                  # "tokenizer" => "path_hierarchy",
+            },
+            # This could also be considered a path_hierarchy
+            'mime_type'  => { type => "string", index => 'not_analyzed' }, # text/html etc.
             "creation_date"    => {
               "type"  =>  "date",
               "format" => "yyyy-MM-dd kk:mm:ss", # yay for Joda, yet-another-timeparser-format
@@ -118,6 +127,7 @@ sub find_or_create_index {
                 #warn "Creating";
                 my $mapping = create_mapping($analyzers{$lang});
                 #warn Dumper $mapping;
+                my @typemap = map { $_ => $mapping } @types;
                 $e->indices->create(index=>$full_name,
                     body => {
                     settings => {
@@ -130,7 +140,9 @@ sub find_or_create_index {
                     "mappings" => {
                         # Hier muessen/sollten wir wir die einzelnen Typen definieren
                         # https://www.elastic.co/guide/en/elasticsearch/reference/current/mapping.html
-                        $type => $mapping,
+                        #$type => $mapping,
+                        # One schema fits all
+                        @typemap,       
                     },
                 })->then(sub {
                     my( $created ) = @_;
